# Copyright 2025 Canonical Ltd.
# See LICENSE file for licensing details.
from litmus_libs.models import DatabaseConfig


def test_database_config_ignores_extra_args():
    # roundtrip test
    base = {"uris": "foo", "username": "bar", "password": "baz"}
    extras = {"somethingelse": "qux", "somethingyetelse": "quz"}
    assert DatabaseConfig(**base, **extras).model_dump() == base



# Copyright 2025 Canonical Ltd.
# See LICENSE file for licensing details.
from unittest.mock import Mock

import pytest
from ops.testing import Relation

from litmus_libs.models import TLSConfigData


@pytest.fixture(scope="function")
def litmus_auth():
    return Relation("litmus-auth")


@pytest.fixture
def workload_container():
    return Mock()


@pytest.fixture
def tls_config():
    return TLSConfigData(
        server_cert="test_cert",
        private_key="test_key",
        ca_cert="test_ca",
    )



# Copyright 2025 Canonical Ltd.
# See LICENSE file for licensing details.
import pytest
from ops import CharmBase
from ops.testing import Context, State
from scenario import Relation

from litmus_libs.interfaces.self_monitoring import SelfMonitoring


class MyCharm(CharmBase):
    META = {
        "name": "echo",
        "requires": {
            "ch-tracing": {"interface": "tracing"},
            "logging": {"interface": "loki_push_api"},
            "cert-transfer": {"interface": "certificate_transfer"},
        },
    }

    def __init__(self, *a, **kw):
        super().__init__(*a, **kw)
        self._self_monitoring = SelfMonitoring(
            self,
            endpoint_overrides={
                "charm-tracing": "ch-tracing",  # non-default!
            },
        )


@pytest.mark.parametrize(
    "bad_meta",
    (
        {
            "name": "gecko",
            # missing required charm-tracing
            "requires": {"logging": {"interface": "loki_push_api"}},
        },
        {
            "name": "gecko",
            # logging has bad interface name
            "requires": {
                "logging": {"interface": "something-weird"},
                "charm-tracing": {"interface": "tracing"},
            },
        },
    ),
)
def test_init_fails_if_bad_meta(bad_meta):
    with pytest.raises(Exception):
        ctx = Context(MyCharm, meta=bad_meta)
        ctx.run(ctx.on.update_status(), State())


@pytest.fixture
def ctx():
    return Context(MyCharm, meta=MyCharm.META)


def test_tracing_integration(ctx):
    # GIVEN a tracing relation
    tracing_relation = Relation("ch-tracing")
    state = State(leader=True, relations=[tracing_relation])
    # WHEN we receive any event
    state_out = ctx.run(ctx.on.update_status(), state)
    # THEN we have published our requested endpoints
    assert "receivers" in state_out.get_relation(tracing_relation.id).local_app_data

    # GIVEN a tracing relation with remote data
    tracing_relation = Relation(
        "ch-tracing",
        remote_app_data={
            "receivers": '[{{"protocol": {{"name": "otlp_grpc", "type": "grpc"}}, "url": "hostname:4317"}}, '
            '{{"protocol": {{"name": "otlp_http", "type": "http"}}, "url": "http://hostname:4318"}}, '
            '{{"protocol": {{"name": "zipkin", "type": "http"}}, "url": "http://hostname:9411" }}]',
        },
    )
    state = State(leader=True, relations=[tracing_relation])
    # WHEN we receive any event, the charm doesn't error out
    ctx.run(ctx.on.update_status(), state)


def test_logging_integration(ctx):
    # GIVEN a logging relation with remote data
    logging_relation = Relation(
        "logging",
        remote_app_data={},
    )
    state = State(leader=True, relations=[logging_relation])
    # WHEN we receive any event
    state_out = ctx.run(ctx.on.update_status(), state)

    # THEN all sidecars get injected with a log-forwarding layer
    for container_out in state_out.containers:
        assert container_out.plan.services["log-forwarding"]



# Copyright 2025 Canonical Ltd.
# See LICENSE file for licensing details.

"""Litmus auth integration endpoint wrapper."""

from typing import Optional

import pydantic

from .base import BaseVersionedModel, SimpleEndpointWrapper


class AuthApiProviderAppDatabagModelV0(BaseVersionedModel):
    """Auth API provider application databag model."""

    version: int = 0
    endpoint: pydantic.HttpUrl


class BackendApiProviderAppDatabagModelV0(BaseVersionedModel):
    """Backend API provider application databag model."""

    version: int = 0
    endpoint: pydantic.HttpUrl


class BackendApiRequirerAppDatabagModelV0(BaseVersionedModel):
    """Backend API requirer application databag model."""

    version: int = 0
    endpoint: pydantic.HttpUrl


class LitmusBackendApiProvider(SimpleEndpointWrapper):
    """Wraps a litmus_backend_http_api provider endpoint.

    Usage:

    ```python
    from typing import Optional
    from litmus_libs.interfaces import LitmusBackendApiProvider

    class LitmusBackendHttpProviderCharm(CharmBase):
        def __init__(self, *args):
            super().__init__(*args)
            self._litmus_backend_api = LitmusBackendApiProvider(
                self.model.get_relation("litmus-backend-http-api"),
                self.app,
            )

        @property
        def _backend_http_api_endpoint(self) -> Optional[str]:
            # Publish the litmus backend http API server endpoint
            url = f"https://{socket.getfqdn()}:1234" # for example
            return self._litmus_backend_api.publish_endpoint(url)

        @property
        def _frontend_url(self):
            # obtain frontend URL from frontend component
            return self._litmus_backend_api.frontend_url()
    ```
    """

    @property
    def frontend_endpoint(self) -> Optional[str]:
        """Retrieve the url of the frontend component."""
        datamodel = self._get(BackendApiRequirerAppDatabagModelV0)
        if not datamodel:
            return None
        return str(datamodel.endpoint)

    def publish_endpoint(
        self,
        endpoint: str,
    ):
        """Publish this backend server's HTTP endpoint to the chaoscenter component."""
        self._set(
            model=BackendApiProviderAppDatabagModelV0,
            data={"endpoint": pydantic.HttpUrl(endpoint)},
        )


class LitmusBackendApiRequirer(SimpleEndpointWrapper):
    """Wraps a litmus_backend_http_api requirer endpoint.

    Usage:

    ```python
    from typing import Optional
    from litmus_libs.interfaces import LitmusBackendApiRequirer

    class LitmusBackendHttpRequirerCharm(CharmBase):
        def __init__(self, *args):
            super().__init__(*args)
            self._litmus_backend_api = LitmusBackendApiRequirer(
                self.model.get_relation("litmus-backend-http-api"),
                self.app,
            )

        @property
        def _backend_http_api_endpoint(self) -> Optional[str]:
            # Get the litmus backend http API server endpoint
            return self._litmus_backend_api.endpoint
    ```
    """

    @property
    def backend_endpoint(self) -> Optional[str]:
        """Fetch the backend API endpoint from relation data."""
        datamodel = self._get(BackendApiProviderAppDatabagModelV0)
        if not datamodel:
            return None
        return str(datamodel.endpoint)

    def publish_endpoint(
        self,
        endpoint: str,
    ):
        """Publish this frontend's HTTP endpoint to the backend component."""
        self._set(
            model=BackendApiRequirerAppDatabagModelV0,
            data={"endpoint": pydantic.HttpUrl(endpoint)},
        )


class LitmusAuthApiProvider(SimpleEndpointWrapper):
    """Wraps a litmus_auth_http_api provider endpoint.

    Usage:

    ```python
    from typing import Optional
    from litmus_libs.interfaces import LitmusAuthApiProvider

    class LitmusAuthHttpProviderCharm(CharmBase):
        def __init__(self, *args):
            super().__init__(*args)
            self._litmus_auth_api = LitmusAuthApiProvider(
                self.model.get_relation("litmus-auth-http-api"),
                self.app,
            )

        @property
        def _auth_http_api_endpoint(self) -> Optional[str]:
            # Publish the litmus auth http API server endpoint
            url = f"https://{socket.getfqdn()}:1234" # for example
            return self._litmus_auth_api.publish_endpoint(url)
    ```
    """

    def publish_endpoint(
        self,
        endpoint: str,
    ):
        """Publish this auth server's HTTP endpoint to the chaoscenter component."""
        self._set(
            model=AuthApiProviderAppDatabagModelV0, data={"endpoint": pydantic.HttpUrl(endpoint)}
        )


class LitmusAuthApiRequirer(SimpleEndpointWrapper):
    """Wraps a litmus_auth_http_api requirer endpoint.

    Usage:

    ```python
    from typing import Optional
    from litmus_libs.interfaces import LitmusAuthApiRequirer

    class LitmusAuthHttpRequirerCharm(CharmBase):
        def __init__(self, *args):
            super().__init__(*args)
            self._litmus_auth_api = LitmusAuthApiRequirer(
                self.model.get_relation("litmus-auth-http-api"),
                self.app,
            )

        @property
        def _auth_http_api_endpoint(self) -> Optional[str]:
            # Get the litmus auth http API server endpoint
            return self._litmus_auth_api.endpoint
    ```
    """

    @property
    def auth_endpoint(self) -> Optional[str]:
        """Fetch the auth API endpoint from relation data."""
        datamodel = self._get(AuthApiProviderAppDatabagModelV0)
        if not datamodel:
            return None
        return str(datamodel.endpoint)



# Copyright 2025 Canonical Ltd.
# See LICENSE file for licensing details.
import dataclasses
import json

import pytest
from ops import CharmBase
from ops.testing import Context, State

from litmus_libs.interfaces.base import VersionMismatchError
from litmus_libs.interfaces.litmus_auth import (
    Endpoint,
    LitmusAuthProvider,
    LitmusAuthRequirer,
)


@pytest.mark.parametrize(
    "input, expected",
    (
        (
            Endpoint(
                grpc_server_host="host",
                grpc_server_port=80,
            ),
            {
                "grpc_server_host": json.dumps("host"),
                "grpc_server_port": json.dumps(80),
                "insecure": json.dumps(False),
                "version": json.dumps(0),
            },
        ),
        (
            Endpoint(
                grpc_server_host="host",
                grpc_server_port=80,
                insecure=True,
            ),
            {
                "grpc_server_host": json.dumps("host"),
                "grpc_server_port": json.dumps(80),
                "insecure": json.dumps(True),
                "version": json.dumps(0),
            },
        ),
    ),
)
def test_provider_publish_endpoint(litmus_auth, input, expected):
    # GIVEN a charm that provides litmus-auth
    ctx = Context(
        CharmBase,
        meta={"name": "provider", "provides": {"litmus-auth": {"interface": "litmus_auth"}}},
    )
    with ctx(
        state=State(
            relations={litmus_auth},
            leader=True,
        ),
        event=ctx.on.update_status(),
    ) as mgr:
        charm = mgr.charm
        # WHEN the charm publishes its endpoint
        provider = LitmusAuthProvider(
            charm.model.get_relation("litmus-auth"),
            charm.app,
        )
        provider.publish_endpoint(input)

        # THEN the local app databag gets populated with the expected content
        state_out = mgr.run()
        databag = state_out.get_relation(litmus_auth.id).local_app_data
        assert databag == expected


@pytest.mark.parametrize(
    "remote_databag, expected",
    (
        (
            {},
            None,
        ),
        (
            {
                "grpc_server_host": '"host"',
                "grpc_server_port": "80",
                "insecure": "false",
                "version": "0",
            },
            Endpoint(grpc_server_host="host", grpc_server_port=80, insecure=False),
        ),
    ),
)
def test_provider_get_backend_grpc_endpoint(litmus_auth, remote_databag, expected):
    # GIVEN a charm that provides litmus-auth
    ctx = Context(
        CharmBase,
        meta={"name": "provider", "provides": {"litmus-auth": {"interface": "litmus_auth"}}},
    )
    with ctx(
        # AND remote has published its endpoint to the databag
        state=State(
            relations={dataclasses.replace(litmus_auth, remote_app_data=remote_databag)},
            leader=True,
        ),
        event=ctx.on.update_status(),
    ) as mgr:
        charm = mgr.charm
        provider = LitmusAuthProvider(
            charm.model.get_relation("litmus-auth"),
            charm.app,
        )
        assert provider.get_backend_grpc_endpoint() == expected


@pytest.mark.parametrize(
    "input, expected",
    (
        (
            Endpoint(grpc_server_host="host", grpc_server_port=80, insecure=False),
            {
                "grpc_server_host": '"host"',
                "grpc_server_port": "80",
                "insecure": "false",
                "version": "0",
            },
        ),
    ),
)
def test_requirer_publish_endpoint(litmus_auth, input, expected):
    # GIVEN a charm that requires litmus-auth
    ctx = Context(
        CharmBase,
        meta={"name": "requirer", "requires": {"litmus-auth": {"interface": "litmus_auth"}}},
    )
    with ctx(
        state=State(
            relations={litmus_auth},
            leader=True,
        ),
        event=ctx.on.update_status(),
    ) as mgr:
        charm = mgr.charm
        requirer = LitmusAuthRequirer(
            charm.model.get_relation("litmus-auth"),
            charm.app,
        )
        # WHEN the requirer publishes its endpoint
        requirer.publish_endpoint(input)

        # THEN the local app databag is populated as expected
        state_out = mgr.run()
        databag = state_out.get_relation(litmus_auth.id).local_app_data
        assert databag == expected


@pytest.mark.parametrize(
    "remote_databag, expected",
    (
        (
            {},
            None,
        ),
        (
            {
                "grpc_server_host": '"host"',
                "grpc_server_port": "80",
                "insecure": "false",
                "version": "0",
            },
            Endpoint(
                grpc_server_host="host",
                grpc_server_port=80,
                insecure=False,
            ),
        ),
        (
            {
                "grpc_server_host": '"host"',
                "grpc_server_port": "80",
                "insecure": "true",
                "version": "0",
            },
            Endpoint(
                grpc_server_host="host",
                grpc_server_port=80,
                insecure=True,
            ),
        ),
    ),
)
def test_requirer_get_auth_grpc_endpoint(litmus_auth, remote_databag, expected):
    # GIVEN a charm that requires litmus-auth
    ctx = Context(
        CharmBase,
        meta={"name": "requirer", "requires": {"litmus-auth": {"interface": "litmus_auth"}}},
    )
    with ctx(
        # AND remote has published its endpoint to the databag
        state=State(
            relations={dataclasses.replace(litmus_auth, remote_app_data=remote_databag)},
            leader=True,
        ),
        event=ctx.on.update_status(),
    ) as mgr:
        charm = mgr.charm
        requirer = LitmusAuthRequirer(
            charm.model.get_relation("litmus-auth"),
            charm.app,
        )
        # WHEN the requirer gets the published endpoint
        endpoint = requirer.get_auth_grpc_endpoint()
        # THEN the fetched data is the same as expected
        assert endpoint == expected


def test_fail_version_mismatch(litmus_auth):
    # GIVEN a charm that provides litmus-auth
    ctx = Context(
        CharmBase,
        meta={"name": "provider", "provides": {"litmus-auth": {"interface": "litmus_auth"}}},
    )

    # WHEN the charm receives its endpoints
    with ctx(
        ctx.on.update_status(),
        state=State(
            relations={
                dataclasses.replace(
                    litmus_auth,
                    remote_app_data={
                        "version": json.dumps(42),
                        "foo": '"bar"',
                    },
                )
            },
            leader=True,
        ),
    ) as mgr:
        charm = mgr.charm
        provider = LitmusAuthProvider(
            charm.model.get_relation("litmus-auth"),
            charm.app,
        )
        # THEN the charm raises an exception if it receives a version the lib doesn't support
        with pytest.raises(VersionMismatchError):
            assert provider.get_backend_grpc_endpoint()



# Copyright 2025 Canonical Ltd.
# See LICENSE file for licensing details.

from unittest.mock import Mock

from litmus_libs.tls_reconciler import TlsReconciler

SERVER_CERT_PATH = "/etc/tls/tls.crt"
PRIVATE_KEY_PATH = "/etc/tls/tls.key"
CA_CERT_PATH = "/usr/local/share/ca-certificates/ca.crt"


def test_reconcile_tls_config_not_called_if_cant_connect_to_workload_container(
    workload_container, tls_config
):
    tls = TlsReconciler(
        container=workload_container,
        tls_cert_path=SERVER_CERT_PATH,
        tls_key_path=PRIVATE_KEY_PATH,
        tls_ca_path=CA_CERT_PATH,
        tls_config_getter=lambda: tls_config,
    )
    tls._reconcile_tls_config = Mock()

    # GIVEN a workload container which can't be connected to
    workload_container.can_connect.return_value = False

    # WHEN tls.reconcile()
    tls.reconcile()

    # THEN _reconcile_tls_config() in not called
    tls._reconcile_tls_config.assert_not_called()


def test_certs_pushed_to_container_if_stored_certs_are_outdated(workload_container, tls_config):
    tls = TlsReconciler(
        container=workload_container,
        tls_cert_path=SERVER_CERT_PATH,
        tls_key_path=PRIVATE_KEY_PATH,
        tls_ca_path=CA_CERT_PATH,
        tls_config_getter=lambda: tls_config,
    )

    # GIVEN a workload container with outdated content
    workload_container.exists.side_effect = lambda path: True
    workload_container.pull.side_effect = lambda path: Mock(read=lambda: "outdated")

    # WHEN _configure_tls called with new TLS config
    tls._configure_tls(
        server_cert=tls_config.server_cert,
        private_key=tls_config.private_key,
        ca_cert=tls_config.ca_cert,
    )

    # THEN certs are updated in the workload container
    workload_container.push.assert_any_call(SERVER_CERT_PATH, "test_cert", make_dirs=True)
    workload_container.push.assert_any_call(PRIVATE_KEY_PATH, "test_key", make_dirs=True)
    workload_container.push.assert_any_call(CA_CERT_PATH, "test_ca", make_dirs=True)


def test_update_ca_certificates_is_run_after_certificates_are_pushed_to_workload_container(
    workload_container, tls_config
):
    tls = TlsReconciler(
        container=workload_container,
        tls_cert_path=SERVER_CERT_PATH,
        tls_key_path=PRIVATE_KEY_PATH,
        tls_ca_path=CA_CERT_PATH,
        tls_config_getter=lambda: tls_config,
    )

    # GIVEN a workload container with outdated content
    workload_container.exists.side_effect = lambda path: False

    # WHEN _configure_tls called with new TLS config
    tls._configure_tls(
        server_cert=tls_config.server_cert,
        private_key=tls_config.private_key,
        ca_cert=tls_config.ca_cert,
    )

    # THEN update-ca-certificates is run in the workload container
    workload_container.exec.assert_called_once_with(["update-ca-certificates", "--fresh"])


def test_certs_not_pushed_to_container_if_stored_certs_are_up_to_date(
    workload_container, tls_config
):
    tls = TlsReconciler(
        container=workload_container,
        tls_cert_path=SERVER_CERT_PATH,
        tls_key_path=PRIVATE_KEY_PATH,
        tls_ca_path=CA_CERT_PATH,
        tls_config_getter=lambda: tls_config,
    )

    # GIVEN a workload container with up-to-date content
    workload_container.exists.side_effect = lambda path: True
    workload_container.pull.side_effect = lambda path: Mock(
        read=lambda: {
            SERVER_CERT_PATH: "test_cert",
            PRIVATE_KEY_PATH: "test_key",
            CA_CERT_PATH: "test_ca",
        }[path]
    )

    # WHEN _configure_tls called with the same TLS config
    tls._configure_tls(
        server_cert=tls_config.server_cert,
        private_key=tls_config.private_key,
        ca_cert=tls_config.ca_cert,
    )

    # THEN certs are not updated in the workload container
    workload_container.push.assert_not_called()


def test_configure_tls_is_called_if_valid_tls_config_is_provider(workload_container, tls_config):
    # GIVEN valid TLS config
    tls = TlsReconciler(
        container=workload_container,
        tls_cert_path=SERVER_CERT_PATH,
        tls_key_path=PRIVATE_KEY_PATH,
        tls_ca_path=CA_CERT_PATH,
        tls_config_getter=lambda: tls_config,
    )
    tls._configure_tls = Mock()

    # WHEN _reconcile_tls_config is called
    tls._reconcile_tls_config()

    # THEN _configure_tls is called with provided TLS config
    tls._configure_tls.assert_called_once_with(
        server_cert=tls_config.server_cert,
        private_key=tls_config.private_key,
        ca_cert=tls_config.ca_cert,
    )


def test_delete_certificates_called_if_tls_config_is_none(workload_container):
    # GIVEN TLS config is None
    tls = TlsReconciler(
        container=workload_container,
        tls_cert_path=SERVER_CERT_PATH,
        tls_key_path=PRIVATE_KEY_PATH,
        tls_ca_path=CA_CERT_PATH,
        tls_config_getter=lambda: None,
    )
    tls._configure_tls = Mock()
    tls._delete_certificates = Mock()

    # WHEN _reconcile_tls_config is called
    tls._reconcile_tls_config()

    # THEN _delete_certificates is called to remove TLS config from the workload container
    tls._delete_certificates.assert_called_once()



# Copyright 2025 Canonical
# See LICENSE file for licensing details.

"""Database Config class."""

from dataclasses import dataclass

from pydantic import BaseModel, ConfigDict


class DatabaseConfig(BaseModel):
    """Model for database client relation databag.

    We need this because the mongo charm lib doesn't expose a typed data structure, so we wrap their API with this instead of passing around raw dicts.
    """

    model_config = ConfigDict(extra="ignore")

    uris: str
    username: str
    password: str


@dataclass
class TLSConfigData:
    """Data structure encapsulating the TLS configuration received over the `tls-certificates` interface."""

    server_cert: str
    private_key: str
    ca_cert: str



# Copyright 2025 Canonical Ltd.
# See LICENSE file for licensing details.

"""Litmus auth integration endpoint wrapper."""

import logging
from dataclasses import asdict, dataclass
from typing import Optional

import pydantic

from .base import BaseVersionedModel, SimpleEndpointWrapper

logger = logging.getLogger()


@dataclass
class Endpoint:
    """User-facing data model representing a gRPC server endpoint."""

    grpc_server_host: str
    grpc_server_port: int
    insecure: bool = False


class _EndpointModel(pydantic.BaseModel):
    grpc_server_host: str
    grpc_server_port: int
    insecure: bool = False


class _LitmusAuthProviderAppDatabagModelV0(BaseVersionedModel, _EndpointModel):
    """V0 provider application databag model for the litmus_auth interface."""

    version: int = 0


class _LitmusAuthRequirerAppDatabagModelV0(BaseVersionedModel, _EndpointModel):
    """V0 requirer application databag model for the litmus_auth interface."""

    version: int = 0


class LitmusAuthProvider(SimpleEndpointWrapper):
    """Wraps a litmus_auth provider endpoint.

    Usage example:
        ```python
        # In your provider's charm code
        from typing import Optional
        from litmus_libs.interfaces.limtus_auth import LitmusAuthProvider, Endpoint

        class LitmusAuthProviderCharm(CharmBase):
            def __init__(self, *args):
                super().__init__(*args)
                self._litmus_auth = LitmusAuthProvider(
                    self.model.get_relation("litmus-auth"),
                    self.app,
                )

            @property
            def _backend_grpc_endpoint(self) -> Optional[Endpoint]:
                # Get the litmus backend gRPC server endpoint
                return self._litmus_auth.get_backend_grpc_endpoint()

            def _publish_auth_grpc_endpoint(self):
                # Publish the litmus auth server endpoint to the litmus backend
                self._litmus_auth.publish_endpoint(Endpoint(
                    grpc_server_host="my-host",
                    grpc_server_port=80,
                ))
        ```
    """

    def publish_endpoint(
        self,
        endpoint: Endpoint,
    ):
        """Publish this auth server's gRPC endpoint to the backend server.

        Raises:
            pydantic.ValidationError: If the provided data does not conform to the expected schema.
        """
        self._set(_LitmusAuthProviderAppDatabagModelV0, asdict(endpoint))

    def get_backend_grpc_endpoint(self) -> Optional[Endpoint]:
        """Get the backend server's gRPC endpoint.

        Raises:
            VersionMismatchError: If the schema version in the relation data
            is not the one supported by this library version.
        """
        data = self._get(_LitmusAuthRequirerAppDatabagModelV0)
        # wrap the data in a user-facing model (Endpoint) to hide internal fields like `version`
        return (
            Endpoint(
                grpc_server_host=data.grpc_server_host,
                grpc_server_port=data.grpc_server_port,
                insecure=data.insecure,
            )
            if data
            else None
        )


class LitmusAuthRequirer(SimpleEndpointWrapper):
    """Wraps a litmus_auth requirer endpoint.

    Usage example:
        ```python
        # In your requirer's charm code
        from typing import Optional
        from litmus_libs.interfaces.limtus_auth import LitmusAuthRequirer, Endpoint

        class LitmusAuthRequirerCharm(CharmBase):
            def __init__(self, *args):
                super().__init__(*args)
                self._litmus_auth = LitmusAuthRequirer(
                    self.model.get_relation("litmus-auth"),
                    self.app,
                )

            @property
            def _auth_grpc_endpoint(self) -> Optional[Endpoint]:
                # Get the auth server's gRPC endpoint from the auth server
                return self._litmus_auth.get_auth_grpc_endpoint()

            def _publish_endpoint(self):
                # Publish the litmus backend server's endpoint to the auth server
                self._litmus_auth.publish_endpoint(
                    Endpoint(
                        grpc_server_host="my-host",
                        grpc_server_port=80,
                    )
                )
        ```
    """

    def publish_endpoint(
        self,
        endpoint: Endpoint,
    ):
        """Publish this backend server's gRPC server endpoint to the auth server.

        Raises:
            pydantic.ValidationError: If the provided data does not conform to the expected schema.
        """
        self._set(_LitmusAuthRequirerAppDatabagModelV0, asdict(endpoint))

    def get_auth_grpc_endpoint(self) -> Optional[Endpoint]:
        """Get the auth server's gRPC endpoint.

        Raises:
            VersionMismatchError: If the schema version in the relation data
            is not supported by this library version.
        """
        data = self._get(_LitmusAuthProviderAppDatabagModelV0)
        # wrap the data in a user-facing model (Endpoint) to hide internal fields like `version`
        return (
            Endpoint(
                grpc_server_host=data.grpc_server_host,
                grpc_server_port=data.grpc_server_port,
                insecure=data.insecure,
            )
            if data
            else None
        )
